"use strict";
/*
 * Deepkit Framework
 * Copyright (C) 2021 Deepkit UG, Marc J. Schmidt
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the MIT License.
 *
 * You should have received a copy of the MIT License along with this program.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresPlatform = void 0;
const sql_1 = require("@deepkit/sql");
const postgres_serializer_1 = require("./postgres-serializer");
const type_1 = require("@deepkit/type");
const postgres_schema_parser_1 = require("./postgres-schema-parser");
const sql_filter_builder_1 = require("./sql-filter-builder");
const core_1 = require("@deepkit/core");
const sqlstring_1 = __importDefault(require("sqlstring"));
function escapeLiteral(value) {
    if (value === null || value === undefined)
        return 'null';
    if (value instanceof Date)
        return sqlstring_1.default.escape(value);
    if ('number' === typeof value || 'bigint' === typeof value)
        return String(value);
    if ('string' !== typeof value)
        return escapeLiteral(String(value));
    let hasBackslash = false;
    let escaped = '\'';
    for (let i = 0; i < value.length; i++) {
        const c = value[i];
        if (c === '\'') {
            escaped += c + c;
        }
        else if (c === '\\') {
            escaped += c + c;
            hasBackslash = true;
        }
        else {
            escaped += c;
        }
    }
    escaped += '\'';
    if (hasBackslash) {
        escaped = ' E' + escaped;
    }
    return escaped;
}
class PostgresPlatform extends sql_1.DefaultPlatform {
    constructor() {
        super();
        this.defaultSqlType = 'text';
        this.serializer = postgres_serializer_1.postgresSerializer;
        this.schemaParserType = postgres_schema_parser_1.PostgresSchemaParser;
        this.addType('number', 'double precision');
        this.addType('date', 'timestamp');
        this.addType('boolean', 'boolean');
        this.addType('class', 'jsonb');
        this.addType('array', 'jsonb');
        this.addType('union', 'jsonb');
        this.addType('partial', 'jsonb');
        this.addType('map', 'jsonb');
        this.addType('patch', 'jsonb');
        this.addType('enum', 'jsonb');
        this.addType('uuid', 'uuid');
        this.addBinaryType('bytea');
    }
    getAggregateSelect(tableName, property, func) {
        if (func === 'group_concat') {
            return `array_to_string(array_agg(${tableName}.${this.quoteIdentifier(property.name)}), ',')`;
        }
        return super.getAggregateSelect(tableName, property, func);
    }
    createSqlFilterBuilder(schema, tableName) {
        return new sql_filter_builder_1.PostgreSQLFilterBuilder(schema, tableName, this.serializer, this.quoteValue.bind(this), this.quoteIdentifier.bind(this));
    }
    quoteValue(value) {
        if (!(value instanceof Date) && (core_1.isObject(value) || type_1.isArray(value)))
            return escapeLiteral(JSON.stringify(value));
        if (value instanceof Date)
            return 'TIMESTAMP ' + sqlstring_1.default.escape(value);
        return escapeLiteral(value);
    }
    setColumnType(column, typeProperty) {
        const db = (typeProperty.data['postgres'] || {});
        if (db.type) {
            sql_1.parseType(column, db.type);
            return;
        }
        super.setColumnType(column, typeProperty);
    }
    getColumnDDL(column) {
        const ddl = [];
        ddl.push(this.getIdentifier(column));
        if (column.isAutoIncrement) {
            ddl.push(`SERIAL`);
        }
        else {
            const foreignKey = column.table.getForeignKeyOfLocalColumn(column);
            if (foreignKey) {
                const [foreignPk] = foreignKey.foreignColumns;
                if (foreignPk.isAutoIncrement) {
                    //the foreignPK has `SERIAL` type, which is basically an INTEGER.
                    ddl.push('INTEGER');
                }
                else {
                    ddl.push((foreignPk.type || 'INTEGER') + column.getSizeDefinition());
                }
            }
            else {
                ddl.push((column.type || 'INTEGER') + column.getSizeDefinition());
            }
            ddl.push(column.isNotNull ? this.getNotNullString() : this.getNullString());
            ddl.push(this.getColumnDefaultValueDDL(column));
        }
        return ddl.filter(sql_1.isSet).join(' ');
    }
    getUniqueDDL(unique) {
        return `CONSTRAINT ${this.getIdentifier(unique)} UNIQUE (${this.getColumnListDDL(unique.columns)})`;
    }
    getDropIndexDDL(index) {
        return `DROP CONSTRAINT ${this.getIdentifier(index)}`;
    }
    supportsInlineForeignKey() {
        return false;
    }
    getAutoIncrement() {
        return '';
    }
    getDropTableDDL(table) {
        return `DROP TABLE IF EXISTS ${this.getIdentifier(table)} CASCADE`;
    }
    getUseSchemaDDL(table) {
        if (!table.schemaName)
            return '';
        return `SET search_path TO ${this.quoteIdentifier(table.schemaName)}`;
    }
    getResetSchemaDDL(table) {
        if (!table.schemaName)
            return '';
        return `SET search_path TO public`;
    }
}
exports.PostgresPlatform = PostgresPlatform;
//# sourceMappingURL=postgres-platform.js.map